<?php

namespace Futage\Show\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Futage\Show\Http\Requests\EpisodeRequest;
use Futage\Show\Interfaces\EpisodeRepositoryInterface;
use Futage\Show\Models\Episode;

/**
 * Resource controller class for episode.
 */
class EpisodeResourceController extends BaseController
{

    /**
     * Initialize episode resource controller.
     *
     * @param type EpisodeRepositoryInterface $episode
     *
     * @return null
     */
    public function __construct(EpisodeRepositoryInterface $episode)
    {
        parent::__construct();
        $this->repository = $episode;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Futage\Show\Repositories\Criteria\EpisodeResourceCriteria::class);
    }

    /**
     * Display a list of episode.
     *
     * @return Response
     */
    public function index(EpisodeRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Futage\Show\Repositories\Presenter\EpisodePresenter::class)
                ->$function();
        }

        $episodes = $this->repository->paginate();

        return $this->response->title(trans('show::episode.names'))
            ->view('show::episode.index', true)
            ->data(compact('episodes'))
            ->output();
    }

    /**
     * Display episode.
     *
     * @param Request $request
     * @param Model   $episode
     *
     * @return Response
     */
    public function show(EpisodeRequest $request, Episode $episode)
    {

        if ($episode->exists) {
            $view = 'show::episode.show';
        } else {
            $view = 'show::episode.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('show::episode.name'))
            ->data(compact('episode'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new episode.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(EpisodeRequest $request)
    {

        $episode = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('show::episode.name')) 
            ->view('show::episode.create', true) 
            ->data(compact('episode'))
            ->output();
    }

    /**
     * Create new episode.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(EpisodeRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $episode                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('show::episode.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('show/episode/' . $episode->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/show/episode'))
                ->redirect();
        }

    }

    /**
     * Show episode for editing.
     *
     * @param Request $request
     * @param Model   $episode
     *
     * @return Response
     */
    public function edit(EpisodeRequest $request, Episode $episode)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('show::episode.name'))
            ->view('show::episode.edit', true)
            ->data(compact('episode'))
            ->output();
    }

    /**
     * Update the episode.
     *
     * @param Request $request
     * @param Model   $episode
     *
     * @return Response
     */
    public function update(EpisodeRequest $request, Episode $episode)
    {
        try {
            $attributes = $request->all();

            $episode->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('show::episode.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('show/episode/' . $episode->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('show/episode/' . $episode->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the episode.
     *
     * @param Model   $episode
     *
     * @return Response
     */
    public function destroy(EpisodeRequest $request, Episode $episode)
    {
        try {

            $episode->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('show::episode.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('show/episode/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('show/episode/' . $episode->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple episode.
     *
     * @param Model   $episode
     *
     * @return Response
     */
    public function delete(EpisodeRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('show::episode.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('show/episode'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/show/episode'))
                ->redirect();
        }

    }

    /**
     * Restore deleted episodes.
     *
     * @param Model   $episode
     *
     * @return Response
     */
    public function restore(EpisodeRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('show::episode.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/show/episode'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/show/episode/'))
                ->redirect();
        }

    }

}
